import { Component, Input, HostBinding, Inject, ElementRef, ApplicationRef, ɵɵdefineInjectable, ɵɵinject, ComponentFactoryResolver, INJECTOR, Injectable, Injector, EventEmitter, Directive, PLATFORM_ID, Output, NgModule } from '@angular/core';
import { ColorService } from 'factor-utils';
import { isPlatformBrowser, CommonModule as CommonModule$1 } from '@angular/common';

class AvatarComponent {
    constructor(colorService) {
        this.colorService = colorService;
        this.class = '';
    }
    set src(value) {
        if (value && value.trim() != '') {
            this._src = value;
            let image = new Image();
            image.src = value;
            image.onload = () => {
                if ("decode" in image) {
                    image.decode().then(() => {
                        this.loaded = true;
                    });
                }
                else {
                    console.error('Image.decode not available.');
                }
            };
        }
    }
    set label(value) {
        this._label = value;
        this.initials = this.getInitials(value);
    }
    get backgroundColor() {
        return this.color || this.colorService.hex(this._label);
    }
    get backgroundImage() {
        return this._src ? `url(${this._src})` : '';
    }
    get hostClasses() {
        return [
            'ft-avatar',
            this.class
        ].join(' ');
    }
    ngOnInit() {
    }
    getInitials(value) {
        let allInitials = value.match(/\b\w/g) || [];
        let initials = ((allInitials.shift() || '') + (allInitials.pop() || '')).toUpperCase();
        return initials;
    }
}
AvatarComponent.decorators = [
    { type: Component, args: [{
                selector: 'ft-avatar',
                template: "<div *ngIf=\"!loaded\">{{ initials }}</div>\n",
                styles: [":host{--default-size:var(--size,3rem);display:inline-flex;align-items:center;justify-content:center;color:#fff;background-size:cover;background-repeat:no-repeat;border-radius:calc(var(--default-size) / 2);font-size:calc(var(--default-size) - (var(--default-size) * .6));min-width:var(--default-size);min-height:var(--default-size);cursor:default;-webkit-user-select:none;user-select:none}div{font-size:1em;line-height:1em}"]
            },] }
];
AvatarComponent.ctorParameters = () => [
    { type: ColorService }
];
AvatarComponent.propDecorators = {
    color: [{ type: Input }],
    class: [{ type: Input }],
    src: [{ type: Input }],
    label: [{ type: Input }],
    backgroundColor: [{ type: HostBinding, args: ['style.background-color',] }],
    backgroundImage: [{ type: HostBinding, args: ['style.background-image',] }],
    hostClasses: [{ type: HostBinding, args: ['class',] }]
};

class IconComponent {
    constructor(configuration) {
        this.configuration = configuration;
        this.class = '';
    }
    set collection(collection) {
        this._collection = collection;
        this.update();
    }
    set mode(mode) {
        this._mode = mode;
        this.update();
    }
    set name(name) {
        this._name = name;
        this.update();
    }
    set path(path) {
        this._path = path;
        this.update();
    }
    get hostClasses() {
        return [
            'ft-icon',
            this.class
        ].join(' ');
    }
    ngOnInit() {
        this.update();
    }
    update() {
        // Set the default collection if the mode is external
        if (!this._collection) {
            if (this.configuration.icon && this.configuration.icon.collection) {
                this._collection = this.configuration.icon.collection;
            }
            else if (this.mode === 'external') {
                this._collection = 'icons';
            }
        }
        if (!this._mode) {
            if (this.configuration.icon && this.configuration.icon.mode) {
                this._mode = this.configuration.icon.mode;
            }
            else {
                this._mode = 'external';
            }
        }
        if (this._mode === 'external') {
            // If the icon mode is external
            if (!this._path) {
                if (this.configuration.icon && this.configuration.icon.path) {
                    this._path = this.configuration.icon.path;
                }
                else {
                    this._path = 'assets';
                }
            }
            this.url = `${this._path}/${this._collection}.svg#${this._name}`;
        }
        else {
            // If the icon mode is inline
            const name = this._collection && this._collection !== 'unset' ? `${this._collection}--${this._name}` : this._name;
            this.url = `#${name}`;
        }
    }
}
IconComponent.decorators = [
    { type: Component, args: [{
                selector: 'ft-icon',
                template: "<svg *ngIf=\"!src; else imageTemplate\"><use attr.xlink:href=\"{{ url }}\" attr.href=\"{{ url }}\" /></svg>\n<ng-template #imageTemplate>\n  <img [src]=\"src\" />\n</ng-template>\n",
                styles: [":host{line-height:0;display:inline-block}:host.ft-icon--1,:host[size=\"1\"]{font-size:1rem}:host.ft-icon--2,:host[size=\"2\"]{font-size:1.5rem}:host.ft-icon--3,:host[size=\"3\"]{font-size:2rem}:host.ft-icon--4,:host[size=\"4\"]{font-size:3rem}:host.ft-icon--5,:host[size=\"5\"]{font-size:4.5rem}:host.ft-icon--6,:host[size=\"6\"]{font-size:8rem}:host.ft-icon--7,:host[size=\"7\"]{font-size:16rem}:host.ft-icon--8,:host[size=\"8\"]{font-size:32rem}svg{width:1em;fill:currentColor}img,svg{height:1em;vertical-align:middle}"]
            },] }
];
IconComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: ['FactorCommonConfiguration',] }] }
];
IconComponent.propDecorators = {
    class: [{ type: Input }],
    collection: [{ type: Input }],
    mode: [{ type: Input }],
    name: [{ type: Input }],
    path: [{ type: Input }],
    size: [{ type: Input }],
    src: [{ type: Input }],
    hostClasses: [{ type: HostBinding, args: ['class',] }]
};

class ImageComponent {
    constructor(element) {
        this.element = element;
        this.class = '';
    }
    ngOnInit() {
        if ("IntersectionObserver" in window) {
            let elementObserver = new IntersectionObserver((entries, observer) => {
                entries.forEach((entry) => {
                    if (entry.isIntersecting) {
                        let image = new Image();
                        image.src = this.src;
                        setTimeout(() => {
                            if (!this.shown && !this.error) {
                                this.loading = true;
                            }
                        }, 100);
                        image.onerror = () => {
                            this.error = true;
                            this.loading = false;
                        };
                        image.onload = () => {
                            if ("decode" in image) {
                                image.decode().then(() => {
                                    this.loading = false;
                                    this.shown = true;
                                });
                            }
                            else {
                                this.loading = false;
                                this.shown = true;
                            }
                        };
                        elementObserver.unobserve(this.element.nativeElement);
                    }
                });
            }, {
                rootMargin: "0px 0px 200px 0px"
            });
            elementObserver.observe(this.element.nativeElement);
        }
        else {
            console.error('IntersectionObserver not available.');
            this.loading = false;
            this.shown = true;
        }
    }
    get hostClasses() {
        return [
            'ft-image',
            this.class
        ].join(' ');
    }
}
ImageComponent.decorators = [
    { type: Component, args: [{
                selector: 'ft-image',
                template: "<img *ngIf=\"shown\" [src]=\"src\" />\n<ft-icon name=\"warning\" size=\"2\" *ngIf=\"error\"></ft-icon>\n",
                styles: [":host{display:inline-block;overflow:hidden;display:flex;align-items:center;justify-content:center}:host.ft-image--loading{background-color:rgba(0,0,0,.03);position:relative;overflow:hidden}:host.ft-image--loading:after{content:\"\";display:block;background-color:rgba(0,0,0,.02);position:absolute;top:0;bottom:0;width:100%;height:100%;transform:translateX(0);animation:placeholder-loading 1.5s ease-in-out infinite}:host.ft-image--error{background-color:rgba(255,0,0,.03)}:host.ft-image--error ft-icon{color:var(--danger)}img{position:relative;z-index:1;max-width:100%;max-height:100%;animation:fade-in .3s}@keyframes fade-in{0%{opacity:0}to{opacity:1}}@keyframes placeholder-loading{0%{transform:translateX(-100%)}to{transform:translateX(100%)}}"]
            },] }
];
ImageComponent.ctorParameters = () => [
    { type: ElementRef }
];
ImageComponent.propDecorators = {
    class: [{ type: Input }],
    error: [{ type: HostBinding, args: ['class.ft-image--error',] }],
    loading: [{ type: HostBinding, args: ['class.ft-image--loading',] }],
    src: [{ type: Input }],
    hostClasses: [{ type: HostBinding, args: ['class',] }]
};

class ProgressComponent {
    constructor() {
        this.class = '';
        this.mode = 'indeterminate';
    }
    ngOnInit() {
        this.value = 0;
    }
    get hostClasses() {
        return [
            'ft-progress',
            this.overlay ? 'ft-progress--overlay' : '',
            this.class
        ].join(' ');
    }
}
ProgressComponent.decorators = [
    { type: Component, args: [{
                selector: 'ft-progress',
                template: "<svg [ngStyle]=\"{'--bar-color': color}\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 100 100\" preserveAspectRatio=\"xMidYMid\">\n    <circle class=\"track\" cx=\"50\" cy=\"50\" r=\"40\" />\n    <circle class=\"bar\" [ngClass]=\"mode\" cx=\"50\" cy=\"50\" r=\"40\"\n        [ngStyle]=\"{'stroke-dashoffset': mode == 'determinate' ? 'calc((3.14159265 * 40 * 2 * (100 - '+value+')) / 100)' : null}\">\n    </circle>\n</svg>",
                styles: [":host{--track-color:rgba(0,0,0,0.08);--bar-color:var(--primary);line-height:0;display:inline-block}:host.ft-progress--1,:host[size=\"1\"]{font-size:1rem}:host.ft-progress--2,:host[size=\"2\"]{font-size:1.5rem}:host.ft-progress--3,:host[size=\"3\"]{font-size:2rem}:host.ft-progress--4,:host[size=\"4\"]{font-size:3rem}:host.ft-progress--5,:host[size=\"5\"]{font-size:4.5rem}:host.ft-progress--6,:host[size=\"6\"]{font-size:8rem}:host.ft-progress--7,:host[size=\"7\"]{font-size:16rem}:host.ft-progress--8,:host[size=\"8\"]{font-size:32rem}:host.ft-progress--overlay{position:fixed;display:flex;align-items:center;justify-content:center;pointer-events:none;z-index:1050;top:0;left:0;right:0;bottom:0;font-size:42px}:host.ft-progress--overlay svg{background:hsla(0,0%,100%,.6);-webkit-backdrop-filter:saturate(50%) blur(3px);backdrop-filter:saturate(50%) blur(3px);box-shadow:0 3px 3px -2px rgba(0,0,0,.2),0 3px 4px 0 rgba(0,0,0,.14),0 1px 8px 0 rgba(0,0,0,.12);border-radius:100vh;padding:.3125rem;animation:progress-reveal .2s;animation-fill-mode:forwards}:host.ft-progress--overlay svg .track{display:none}svg{width:1em;height:1em;vertical-align:middle}svg .track{fill:none;stroke-width:10;stroke:var(--track-color)}svg .bar{fill:none;stroke-opacity:.9;stroke-width:6;stroke:var(--bar-color)}svg .bar.indeterminate{animation:progress-rotation 2s linear infinite}svg .bar.determinate{stroke-dasharray:calc(2 * 3.1415926536 * 40)}@keyframes progress-reveal{0%{transform:translateY(-100%);opacity:0}to{transform:translateY(0);opacity:1}}@keyframes progress-rotation{0%{stroke-dashoffset:0;stroke-dasharray:150.6 100.4}50%{stroke-dasharray:1 250}to{stroke-dashoffset:502;stroke-dasharray:150.6 100.4}}"]
            },] }
];
ProgressComponent.ctorParameters = () => [];
ProgressComponent.propDecorators = {
    class: [{ type: Input }],
    color: [{ type: Input }],
    mode: [{ type: Input }],
    overlay: [{ type: Input }],
    size: [{ type: Input }],
    value: [{ type: Input }],
    hostClasses: [{ type: HostBinding, args: ['class',] }]
};

class ProgressService {
    constructor(componentFactoryResolver, injector) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.injector = injector;
    }
    show() {
        if (!this.componentRef) {
            this.appRef = this.injector.get(ApplicationRef);
            // 1. Create a component reference from the component 
            this.componentRef = this.componentFactoryResolver
                .resolveComponentFactory(ProgressComponent)
                .create(this.injector);
            // 2. Attach component to the appRef so that it's inside the ng component tree
            this.appRef.attachView(this.componentRef.hostView);
            this.componentRef.instance.overlay = true;
            // 3. Get DOM element from component
            const domElem = this.componentRef.hostView
                .rootNodes[0];
            // 4. Append DOM element to the body
            document.body.appendChild(domElem);
            return true;
        }
        else {
            return false;
        }
    }
    hide() {
        if (this.componentRef) {
            this.appRef.detachView(this.componentRef.hostView);
            this.componentRef.destroy();
            this.componentRef = null;
            return true;
        }
        else {
            return false;
        }
    }
}
ProgressService.ɵprov = ɵɵdefineInjectable({ factory: function ProgressService_Factory() { return new ProgressService(ɵɵinject(ComponentFactoryResolver), ɵɵinject(INJECTOR)); }, token: ProgressService, providedIn: "root" });
ProgressService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
ProgressService.ctorParameters = () => [
    { type: ComponentFactoryResolver },
    { type: Injector }
];

class ObserveIntersectingDirective {
    constructor(element, platformId) {
        this.element = element;
        this.platformId = platformId;
        this.event = new EventEmitter();
    }
    ngOnInit() {
        if (isPlatformBrowser(this.platformId)) {
            if ("IntersectionObserver" in window) {
                const elementObserver = new IntersectionObserver((entries, observer) => {
                    entries.forEach((entry) => {
                        this.event.emit(entry.isIntersecting);
                    });
                }, this.options);
                elementObserver.observe(this.element.nativeElement);
            }
            else {
                console.error('ftObserveIntersecting not available in this browser.');
            }
        }
    }
}
ObserveIntersectingDirective.decorators = [
    { type: Directive, args: [{
                selector: '[ftObserveIntersecting]'
            },] }
];
ObserveIntersectingDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Object, decorators: [{ type: Inject, args: [PLATFORM_ID,] }] }
];
ObserveIntersectingDirective.propDecorators = {
    options: [{ type: Input, args: ['ftObserveIntersectingOptions',] }],
    event: [{ type: Output, args: ['ftObserveIntersecting',] }]
};

class CommonModule {
    static forRoot(configuration) {
        return {
            ngModule: CommonModule,
            providers: [
                { provide: 'FactorCommonConfiguration', useValue: configuration }
            ]
        };
    }
}
CommonModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    AvatarComponent,
                    IconComponent,
                    ProgressComponent,
                    ImageComponent,
                    ObserveIntersectingDirective
                ],
                imports: [
                    CommonModule$1
                ],
                exports: [
                    AvatarComponent,
                    IconComponent,
                    ImageComponent,
                    ObserveIntersectingDirective,
                    ProgressComponent
                ]
            },] }
];

/*
 * Public API Surface of common
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AvatarComponent, CommonModule, IconComponent, ImageComponent, ObserveIntersectingDirective, ProgressComponent, ProgressService };
//# sourceMappingURL=factor-common.js.map
